﻿# include <cstdlib>
# include <iostream>
# include <fstream>
# include <math.h>
# include <time.h>
# include "mpi.h"
using namespace std;
double initial_condition ( double x, double time );
double boundary_condition ( double time,int flag);
clock_t start,finish;
int main(int argv, char** argc)
{
	int id,m; //m: Количество процессов; id: номер процесса
	MPI_Init(&argv,&argc);// initial mpi
	MPI_Comm_rank(MPI_COMM_WORLD, &id);//Определим номер текущего процесса
    MPI_Comm_size(MPI_COMM_WORLD, &m); //Определим количество процессов
	
	double a=0; //левый конец интервала
	double b=1; //правый конец интервала
	double dx=0.01;   // Шаг по x 
	int idx=(b-a)/m/dx;

	double t_start=0; // начало времени расчета
	double t_end=1; // Конец времени расчета
	double dt=0.0001; // Шаг по времени
	int nt=(t_end-t_start)/dt;// количество шагов по времени 
	
	if(id==0) // Начало отсчета
	{
		start=clock();
	}
	double *T0,*T1;
	double *X;
	T0=new double[idx+3];
	T1=new double[idx+3];
	for(int i=0;i<idx+3;i++) // Initial condition t=0
	{
		T0[i]=StartCond(a+(i-1)*dx + id*idx*dx + id*dx,0);
	}

	for(int j=1;j<nt;j++)
	{

		if(id==0) 
			// Если это первый процесс
		{
			double sendr=T0[idx+1];
			MPI_Send(&sendr,1,MPI_DOUBLE,1,0,MPI_COMM_WORLD);
			double recvr; 
			MPI_Recv(&recvr,1,MPI_DOUBLE,1,0,MPI_COMM_WORLD,MPI_STATUSES_IGNORE);
			T0[idx+2]=recvr;
		}
		else if(id==m-1)
			// Если это последний процесс
		{
			double  sendl=T0[1];
			MPI_Send(&sendl,1,MPI_DOUBLE,id-1,0,MPI_COMM_WORLD);
			double recvl;
			MPI_Recv(&recvl,1,MPI_DOUBLE,id-1,0,MPI_COMM_WORLD,MPI_STATUSES_IGNORE);
			T0[0]=recvl;
		}
		else
		{
			//если это средний процесс
			double  sendl=T0[1];
			MPI_Send(&sendl,1,MPI_DOUBLE,id-1,0,MPI_COMM_WORLD);
			double  recvl;
			MPI_Recv(&recvl,1,MPI_DOUBLE,id-1,0,MPI_COMM_WORLD,MPI_STATUSES_IGNORE);
			T0[0]=recvl;

			double  sendr= T0[idx+1];
			MPI_Send(&sendr,1,MPI_DOUBLE,id+1,0,MPI_COMM_WORLD);
			double recvr;
			MPI_Recv(&recvr,1,MPI_DOUBLE,id+1,0,MPI_COMM_WORLD,MPI_STATUSES_IGNORE);
			T0[idx+2]=recvr;
		}
		for(int i=1;i<idx+2;i++)
		{
			T1[i]=T0[i]+	dt*(T0[i+1]+T0[i-1]-2*T0[i])/dx/dx; // вычисляем температуру на i-том узле
		}
		if(id==0) //Исправим значение температуры для первого процесса с помощью граничного условия
		{
			T1[1]=EndCondit(j*dt,1);
		}
		else if(id==m-1)//Исправим значение температуры для последнего процесса с помощью граничного условия
		{
			T1[idx+1]=EndCondit(j*dt,2);
		}
		for(int i=0;i<idx+3;i++)
		{
			T0[i]=T1[i];
		}
		T0[0]=T0[1];
		T0[idx+2]=T0[idx+1];
	}
	
	if(id==0)  
	{
	  finish=clock();//
	  double processtime =finish-start;
	  printf ("Время расчета %f \n",((float)processtime)/CLOCKS_PER_SEC);
	}
	MPI_Finalize();
	
}
double StartCond ( double x, double time )
{
	double value;
	value=36.6*x;
	return value;
}
double EndCondit ( double t,int p)
{
	double value;
	if(p==1)
	{
		value=cos(2*t)+0.5;
	}
	else if(p==2)
	{
		value=sin(2*t)-0.5;
	}
	return value;
}